<?php
!defined( 'YITH_WCBK' ) && exit; // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Frontend_Action_Handler' ) ) {
    /**
     * Class YITH_WCBK_Frontend_Action_Handler
     * handle all Frontend Actions (as request-confirmation)
     *
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     */
    class YITH_WCBK_Frontend_Action_Handler {

        /**
         * Hook in methods.
         */
        public static function init() {
            add_action( 'wp_loaded', array( __CLASS__, 'request_confirmation_action' ), 90 );

            add_action( 'wp_loaded', array( __CLASS__, 'cancel_booking_action' ), 90 );

            add_action( 'wp_loaded', array( __CLASS__, 'pay_confirmed_booking' ), 90 );
        }

        /**
         * Request confirm action.
         *
         * Checks for a valid request, does validation (via hooks) and then redirects if valid.
         */
        public static function request_confirmation_action() {
            if ( empty( $_REQUEST[ 'booking-request-confirmation' ] ) || !is_numeric( $_REQUEST[ 'booking-request-confirmation' ] ) ) {
                return;
            }

            $product_id = apply_filters( 'yith_wcbk_request_confirmation_product_id', absint( $_REQUEST[ 'booking-request-confirmation' ] ) );
            /**  @var WC_Product_Booking $product */
            $product = wc_get_product( $product_id );

            if ( !$product || !YITH_WCBK_Product_Post_Type_Admin::is_booking( $product_id ) ) {
                return;
            }

            if ( !is_user_logged_in() ) {
                wc_add_notice( __( 'You must log in before asking for a booking confirmation', 'yith-booking-for-woocommerce' ), 'error' );

                return;
            }

            $args                  = $_REQUEST;
            $args[ 'add-to-cart' ] = $product_id;

            $booking_data                    = YITH_WCBK_Cart::get_booking_data_from_request( $args );
            $booking_data[ 'title' ]         = $product->get_title();
            $booking_data[ 'user_id' ]       = get_current_user_id();
            $booking_data[ 'product_id' ]    = $product_id;
            $booking_data[ 'status' ]        = 'bk-pending-confirm';
            $booking_data[ 'duration_unit' ] = $product->get_duration_unit();

            if ( isset( $booking_data[ 'person_types' ] ) ) {
                $booking_data[ 'person_types' ] = yith_wcbk_parse_booking_person_types_array( $booking_data[ 'person_types' ] );
            }

            if ( isset( $booking_data[ 'booking_services' ] ) ) {
                $booking_data[ 'services' ] = $booking_data[ 'booking_services' ];
                unset( $booking_data[ 'booking_services' ] );
            }

            $booking = new YITH_WCBK_Booking( 0, $booking_data );

            if ( $booking->is_valid() ) {
                wc_add_notice( __( 'Request for booking confirmation sent', 'yith-booking-for-woocommerce' ) );

                return;
            } else {
                wc_add_notice( __( 'Error in requesting booking confirmation', 'yith-booking-for-woocommerce' ), 'error' );
            }
        }

        /**
         * Cancel Booking
         */
        public static function cancel_booking_action() {
            if ( empty( $_REQUEST[ 'cancel-booking' ] ) || !is_numeric( $_REQUEST[ 'cancel-booking' ] ) || !is_user_logged_in() ) {
                return;
            }
            $booking_id = absint( $_REQUEST[ 'cancel-booking' ] );
            $booking    = yith_get_booking( $booking_id );
            if ( !$booking || !$booking->is_valid() || $booking->user_id != get_current_user_id() || !$booking->can_be( 'cancelled_by_user' ) ) {
                wc_add_notice( __( 'This booking cannot be cancelled. Contact us for more info', 'yith-booking-for-woocommerce' ), 'error' );

                return;
            }

            $booking->update_status( 'cancelled_by_user' );

            if ( $booking->has_status( 'cancelled' ) ) {
                wc_add_notice( sprintf( __( 'Booking %s cancelled', 'yith-booking-for-woocommerce' ), $booking->get_title() ) );
            } else {
                wc_add_notice( sprintf( __( 'Booking %s not cancelled', 'yith-booking-for-woocommerce' ), $booking->get_title() ), 'error' );
            }

            wp_safe_redirect( $booking->get_view_booking_url() );
            exit;
        }

        /**
         * Pay confirmed Booking
         */
        public static function pay_confirmed_booking() {
            if ( empty( $_REQUEST[ 'pay-confirmed-booking' ] ) || !is_numeric( $_REQUEST[ 'pay-confirmed-booking' ] ) || !is_user_logged_in() ) {
                return;
            }
            $error      = false;
            $booking_id = absint( $_REQUEST[ 'pay-confirmed-booking' ] );
            $booking    = yith_get_booking( $booking_id );

            if ( !$booking || !$booking->is_valid() || $booking->user_id != get_current_user_id() )
                $error = true;

            $product_id = $booking->product_id;
            $product    = wc_get_product( $booking->product_id );
            if ( !$product )
                $error = true;

            if ( $error ) {
                wc_add_notice( __( 'This booking cannot be paid. Contact us for more info', 'yith-booking-for-woocommerce' ), 'error' );

                return;
            }

            $cart = WC()->cart;
            $cart->empty_cart();

            $cart_item_data = array(
                'yith_booking_data' => array(
                    'from'             => $booking->from,
                    'to'               => $booking->to,
                    'duration'         => $booking->get_duration(),
                    'persons'          => $booking->persons,
                    'person_types'     => yith_wcbk_parse_booking_person_types_array( $booking->person_types, true ),
                    'booking_services' => $booking->services,
                    '_booking_id'      => $booking_id
                )
            );

            $cart->add_to_cart( $product_id, 1, 0, array(), $cart_item_data );

            wp_safe_redirect( wc_get_checkout_url() );
            exit;
        }
    }
}